const Me = imports.misc.extensionUtils.getCurrentExtension();
const { Clutter, Gio, GLib, Pango, St } = imports.gi;
const utils = Me.imports.utils;
const LOCAL_PLUGINS = GLib.get_home_dir() + "/.local/share/pop-shell/launcher/";
const SYSTEM_PLUGINS = "/usr/lib/pop-shell/launcher/";
var Response;
(function (Response) {
    function parse(input) {
        try {
            let object = JSON.parse(input);
            switch (object.event) {
                case "close":
                case "fill":
                case "queried":
                    return object;
            }
        }
        catch (e) {
        }
        return null;
    }
    Response.parse = parse;
})(Response || (Response = {}));
var Plugin;
(function (Plugin) {
    function read(file) {
        global.log(`found plugin at ${file}`);
        try {
            let [ok, contents] = Gio.file_new_for_path(file)
                .load_contents(null);
            if (ok)
                return parse(imports.byteArray.toString(contents));
        }
        catch (e) {
        }
        return null;
    }
    Plugin.read = read;
    function parse(input) {
        try {
            return JSON.parse(input);
        }
        catch (e) {
            return null;
        }
    }
    Plugin.parse = parse;
    function listen(plugin) {
        if (!plugin.proc) {
            const proc = Plugin.start(plugin);
            if (proc) {
                plugin.proc = proc;
            }
            else {
                return null;
            }
        }
        try {
            let [bytes,] = plugin.proc.stdout.read_line(null);
            return Response.parse(imports.byteArray.toString(bytes));
        }
        catch (e) {
            return null;
        }
    }
    Plugin.listen = listen;
    function complete(plugin) {
        return send(plugin, { event: "complete" });
    }
    Plugin.complete = complete;
    function query(plugin, value) {
        return send(plugin, { event: "query", value });
    }
    Plugin.query = query;
    function quit(plugin) {
        if (plugin.proc) {
            send(plugin, { event: "quit" });
            plugin.proc = null;
        }
    }
    Plugin.quit = quit;
    function submit(plugin, id) {
        return send(plugin, { event: "submit", id });
    }
    Plugin.submit = submit;
    function send(plugin, event) {
        let string = JSON.stringify(event);
        if (!plugin.proc) {
            plugin.proc = start(plugin);
        }
        function attempt(plugin, string) {
            if (!plugin.proc)
                return false;
            try {
                plugin.proc.stdin.write_bytes(new GLib.Bytes(string + "\n"), null);
                return true;
            }
            catch (e) {
                global.log(`failed to send message to ${plugin.config.name}: ${e}`);
                return false;
            }
        }
        if (!attempt(plugin, string)) {
            plugin.proc = start(plugin);
            if (!attempt(plugin, string))
                return false;
        }
        return true;
    }
    Plugin.send = send;
    function start(plugin) {
        return utils.async_process_ipc([plugin.cmd]);
    }
    Plugin.start = start;
})(Plugin || (Plugin = {}));
var LauncherService = class LauncherService {
    constructor() {
        this.plugins = new Map();
        this.register_plugins();
    }
    destroy() {
        for (const plugin of this.plugins.values())
            Plugin.quit(plugin);
    }
    query(query, callback) {
        for (const plugin of this.match_query(query)) {
            global.log(`Plugin "${plugin.config.name} matches ${query}`);
            if (Plugin.query(plugin, query)) {
                const res = Plugin.listen(plugin);
                if (res)
                    callback(plugin, res);
            }
            else {
                Plugin.quit(plugin);
            }
        }
    }
    stop_services() {
        for (const plugin of this.plugins.values()) {
            Plugin.quit(plugin);
            plugin.proc = null;
        }
    }
    register_plugins() {
        this.register_plugin_directory(LOCAL_PLUGINS);
        this.register_plugin_directory(SYSTEM_PLUGINS);
    }
    register_plugin_directory(directory) {
        global.log(`checking for plugins in ${directory}`);
        let dir = Gio.file_new_for_path(directory);
        if (!dir.query_exists(null))
            return;
        try {
            let entries = dir.enumerate_children('standard::*', Gio.FileQueryInfoFlags.NONE, null);
            let entry;
            while ((entry = entries.next_file(null)) !== null) {
                if (entry.get_file_type() === 2) {
                    let name = entry.get_name();
                    const metapath = directory + '/' + name + '/meta.json';
                    const metadata = Gio.file_new_for_path(metapath);
                    if (!metadata.query_exists(null))
                        continue;
                    let config = Plugin.read(metapath);
                    if (!config || this.plugins.has(config.name))
                        continue;
                    let cmd = directory + '/' + name + '/' + config.exec;
                    global.log(`found plugin: ${config.name}`);
                    let pattern = config.pattern ? new RegExp(config.pattern) : null;
                    this.plugins.set(config.name, { config, cmd, proc: null, pattern });
                }
            }
        }
        catch (e) {
            global.log(`error enumerating: ${e}`);
        }
    }
    *match_query(query) {
        for (const plugin of this.plugins.values()) {
            if (!plugin.pattern || plugin.pattern.test(query)) {
                yield plugin;
            }
            else {
                Plugin.quit(plugin);
            }
        }
    }
}
var SearchOption = class SearchOption {
    constructor(title, description, category_icon, icon, icon_size, id) {
        this.shortcut = new St.Label({ text: "", y_align: Clutter.ActorAlign.CENTER, style: "padding-left: 6px;padding-right: 6px" });
        this.title = title;
        this.description = description;
        this.id = id;
        let cat_icon = new St.Icon({
            icon_name: category_icon,
            icon_size: icon_size / 2,
            style_class: "pop-shell-search-cat"
        });
        let layout = new St.BoxLayout({});
        cat_icon.set_y_align(Clutter.ActorAlign.CENTER);
        let label = new St.Label({ text: title });
        label.clutter_text.set_ellipsize(Pango.EllipsizeMode.END);
        layout.add_child(cat_icon);
        if (icon) {
            let app_icon;
            if ("name" in icon) {
                app_icon = new St.Icon({
                    icon_name: icon.name,
                    icon_size,
                    style_class: "pop-shell-search-icon"
                });
            }
            else if ("gicon" in icon) {
                app_icon = new St.Icon({
                    gicon: icon.gicon,
                    icon_size,
                    style_class: "pop-shell-search-icon"
                });
            }
            else {
                app_icon = icon.widget;
                app_icon.style_class = "pop-shell-search-icon";
            }
            app_icon.set_y_align(Clutter.ActorAlign.CENTER);
            layout.add_child(app_icon);
        }
        let info_box = new St.BoxLayout({ y_align: Clutter.ActorAlign.CENTER, vertical: true, x_expand: true });
        info_box.add_child(label);
        if (description) {
            info_box.add_child(new St.Label({ text: description, style: "font-size: small" }));
        }
        layout.add_child(info_box);
        layout.add_child(this.shortcut);
        this.widget = new St.Button({ style_class: "pop-shell-search-element" });
        this.widget.add_actor(layout);
    }
}
//# sourceMappingURL=launcher.js.map